#**************************************************************************
#   Copyright (C) 2009 by Michael Andreen <harvATruinDOTnu>               *
#                                                                         *
#   This program is free software; you can redistribute it and/or modify  *
#   it under the terms of the GNU General Public License as published by  *
#   the Free Software Foundation; either version 2 of the License, or     *
#   (at your option) any later version.                                   *
#                                                                         *
#   This program is distributed in the hope that it will be useful,       *
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#   GNU General Public License for more details.                          *
#                                                                         *
#   You should have received a copy of the GNU General Public License     *
#   along with this program; if not, write to the                         *
#   Free Software Foundation, Inc.,                                       *
#   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
#**************************************************************************/

package NDIRC::Context;
use strict;
use warnings;
use feature ':5.10';

use Moose;

use Set::Object ();

has host => (
	is => 'ro',
	isa => 'Str',
	required => 1
);

has nick => (
	is => 'ro',
	isa => 'Str',
	required => 1
);

has channel => (
	is => 'ro',
	isa => 'Str',
	required => 1
);

has roles => (
	is => 'ro',
	isa => 'Object',
	lazy_build => 1
);

has uid => (
	is => 'ro',
	isa => 'Int',
	lazy_build => 1
);

has disp => (
	is => 'ro',
	isa => 'Object',
	required => 1
);

has model => (
	is => 'ro',
	isa => 'Object',
	required => 1
);

has server => (
	is => 'ro',
	isa => 'Object',
	required => 1
);

has reply_string => (
	is => 'ro',
	isa => 'Str',
	required => 1,
);

sub assert_user_roles {
	my ($self,@roles) = @_;
	return 1 unless @roles;

	my $need = Set::Object->new(@roles);

	if ($self->roles->superset($need)){
		return 1;
	}

	die "Access denied";
}

sub check_user_roles {
	my ($self,@roles) = @_;

	local $@;
	eval { $self->assert_user_roles(@roles) };
}

sub reply {
	my ($self,$msg) = @_;

	my @command = split / /, $self->reply_string;
	$self->message(@command, $msg);
}

sub message {
	my ($self,$command, $target, $msg) = @_;

	$msg =~ s`<b>(.*?)</b>`${\(chr(2))}$1${\(chr(15))}`gi;
	$msg =~ s`<c(\d+)>(.*?)</c>`${\(chr(3))}$1$2${\(chr(15))}`gi;

	#Split the message, using the, slightly modified, algorithm from splitlong.pl in the irssi distribution.
	if ($command eq 'privmsg'){
		my $lend = ' ...';
		my $lstart = '... ';
		my $maxlength = $self->server->{msg_length} - bytes::length("privmsg $target :" . $self->server->nick_name());
		my $maxlength2 = $maxlength - bytes::length($lend);

		if (bytes::length($msg) > ($maxlength)) {
			my @spltarr;

			while (bytes::length($msg) > ($maxlength)) {
				my $pos = rindex($msg, " ", $maxlength2);
				push @spltarr, substr($msg, 0, ($pos < ($maxlength/10 + 4)) ? $maxlength2  : $pos)  . $lend;
				$msg = $lstart . substr($msg, ($pos < ($maxlength/10 + 4)) ? $maxlength2 : $pos+1);
			}

			push @spltarr, $msg;
			for (@spltarr) {
				$self->command($command, $target, $_);
			}
			return;
		}
	}
	$self->command($command, $target, $msg);
}

sub command {
	my ($self,@command) = @_;

	$self->server->yield(@command);
}

sub intel_log {
	my ($c,$planet, $message) = @_;
	my $log = $c->model->prepare_cached(q{
INSERT INTO forum_posts (ftid,uid,message) VALUES(
	(SELECT ftid FROM planets WHERE pid = $3),$1,$2)
		});
	$log->execute($c->uid,$message,$planet);
}

sub def_log {
	my ($c,$call, $message) = @_;
	my $log = $c->model->prepare(q{
INSERT INTO forum_posts (ftid,uid,message) VALUES(
	(SELECT ftid FROM calls WHERE call = $3),$1,$2)
		});
	$log->execute($c->uid,$message,$call);
}

sub _build_roles {
	my ($self) = @_;

	my $query = $self->model->prepare(q{
SELECT role FROM group_roles
WHERE gid IN (SELECT gid FROM groupmembers JOIN users USING (uid)
	WHERE hostmask = $1)
		});
	$query->execute($self->host);

	my @roles;
	while (my $group = $query->fetchrow_hashref){
		push @roles,$group->{role};
	}
	return Set::Object->new(@roles);
}

sub _build_uid {
	my ($self) = @_;

	my $query = $self->model->prepare(q{
SELECT uid FROM users
WHERE hostmask = $1
		});
	$query->execute($self->host);

	if (my ($uid) = $query->fetchrow_array){
		$query->finish;
		return $uid;
	}
	return -4;
}

1;
